/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2013 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.archive.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.NotThreadSafe;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.nio.file.Path;
import org.joda.time.DateTime;
import it.tidalwave.util.spi.FinderSupport;
import it.tidalwave.integritychecker.archive.Scan;
import it.tidalwave.integritychecker.archive.ScannedFile;
import lombok.Getter;

/***********************************************************************************************************************
 *
 * A default implementation of {@link Scan}.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@NotThreadSafe
public class DefaultScan implements Scan
  {
    /*******************************************************************************************************************
     * 
     * 
     ******************************************************************************************************************/
    class DefaultScannedFileFinder extends FinderSupport<ScannedFile, ScannedFileFinder> implements ScannedFileFinder
      {
        @CheckForNull
        private Path path;
        
        @Override @Nonnull
        public DefaultScannedFileFinder clone()
          {
            final DefaultScannedFileFinder clone = (DefaultScannedFileFinder)super.clone();
            clone.path = this.path;
            return clone;
          }

        @Override @Nonnull
        public ScannedFileFinder withPath (final @Nonnull Path path) 
          {
            final DefaultScannedFileFinder clone = clone();
            clone.path = path;
            return clone;
          }
        
        @Override @Nonnull
        protected List<? extends ScannedFile> computeNeededResults()
          {
            final List<ScannedFile> result = new ArrayList<>();
            
            if (path != null)
              {
                final ScannedFile scannedFile = scannedFileMapByPath.get(path);
                
                if (scannedFile != null)
                  {
                    result.add(scannedFile);  
                  }
              }
            else
              {
                result.addAll(scannedFileMapByPath.values());
              } 
            return result;
          }
      }
    
    private final Map<Path, ScannedFile> scannedFileMapByPath = new TreeMap<>();
    
    @Getter @Nonnull
    private final DateTime beginScanDateTime = new DateTime();
    
    @Getter @Nonnull
    private DateTime endScanDateTime = new DateTime();
    
    /*******************************************************************************************************************
     * 
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override @Nonnull
    public ScannedFile.Builder addScannedFile()
      {
        return new ScannedFile.Builder()
          {
            @Override
            protected void register (final @Nonnull ScannedFile scannedFile)
              {
                scannedFileMapByPath.put(scannedFile.getFile().getFileName(), scannedFile);
                endScanDateTime = new DateTime();
              }
          };
      }
    
    /*******************************************************************************************************************
     * 
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override @Nonnull
    public ScannedFileFinder find()
      {
        return new DefaultScannedFileFinder();
      }
  }
