/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2013 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.archive.impl;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.joda.time.DateTime;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.Finder;
import it.tidalwave.util.spi.FinderSupport;
import it.tidalwave.integritychecker.archive.Scan;
import it.tidalwave.integritychecker.archive.ScanArchive;

/***********************************************************************************************************************
 *
 * The whole container of all {@link Scan}s.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=ScanArchive.class)
public class DefaultScanArchive implements ScanArchive 
  {
    /*******************************************************************************************************************
     * 
     * 
     ******************************************************************************************************************/
    class DefaultScanFinder extends FinderSupport<Scan, ScanFinder> implements ScanFinder
      {
        @CheckForNull
        private DateTime dateTime;
        
        @Override @Nonnull
        public DefaultScanFinder clone()
          {
            final DefaultScanFinder clone = (DefaultScanFinder)super.clone();
            clone.dateTime = this.dateTime;
            return clone;
          }

        @Override @Nonnull
        public ScanFinder withDateTime (@Nonnull DateTime dateTime)
          {
            final DefaultScanFinder clone = clone();
            clone.dateTime = dateTime;
            return clone;
          }
        
        @Override @Nonnull
        protected List<? extends Scan> computeNeededResults()
          {
            final List<Scan> result = new ArrayList<>();
            
            if (withDateTime(dateTime) != null)
              {
                final Scan scan = scanMapByDateTime.get(dateTime);
                
                if (scan != null)
                  {
                    result.add(scan);  
                  }
              }
            else
              {
                result.addAll(scanMapByDateTime.values());
              } 
            
            return result;
          }
      }
    
    private final Map<DateTime, Scan> scanMapByDateTime = new HashMap<>();
    
    /*******************************************************************************************************************
     * 
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override
    public void addScan (final @Nonnull Scan scan)
      {
        scanMapByDateTime.put(scan.getBeginScanDateTime(), scan);
      }
    
    /*******************************************************************************************************************
     * 
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override @Nonnull
    public Finder<Scan> find()
      {
        return new DefaultScanFinder(); 
      }
  }
